# Generated by roxygen2: do not edit by hand

export(arrangeDS)
export(asTibbleDS)
export(bindColsDS)
export(bindRowsDS)
export(caseWhenDS)
export(checkPermissivePrivacyControlLevel)
export(distinctDS)
export(filterDS)
export(groupByDS)
export(groupKeysDS)
export(ifElseDS)
export(listPermittedTidyverseFunctionsDS)
export(mutateDS)
export(renameDS)
export(selectDS)
export(sliceDS)
export(ungroupDS)
importFrom(cli,cli_abort)
importFrom(dplyr,desc)
importFrom(rlang,eval_tidy)
importFrom(rlang,parse_expr)
