% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupKeysDS.R
\name{groupKeysDS}
\alias{groupKeysDS}
\title{Performs dplyr \code{group_keys}.}
\usage{
groupKeysDS(tidy_select, x)
}
\arguments{
\item{tidy_select}{Unused in this function.}

\item{x}{a grouped tibble.}
}
\value{
A data frame describing the groups.
}
\description{
DataSHIELD implementation of \code{dplyr::group_keys}
}
